// OSerialPort.cpp: implementation of the OSerialPort class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "OSerialPort.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

const BOOL EmulMyDevice = FALSE;

#include <math.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

OSerialPort::OSerialPort()
{
  m_hFile = INVALID_HANDLE_VALUE;  
};

OSerialPort::~OSerialPort()
{
  ClosePort();
};

bool OSerialPort::OpenPort(int nPort, int Speed)
{
  ClosePort();
  
  CString s;
  s.Format("COM%d:", nPort);
  if (nPort>9 || 1) s.Format("\\\\.\\COM%d", nPort);   

  m_hFile = CreateFile(s, GENERIC_WRITE | GENERIC_READ, 0, NULL, OPEN_EXISTING, /*FILE_FLAG_OVERLAPPED*/0, NULL);
  if (m_hFile==INVALID_HANDLE_VALUE) 
  {
    DWORD LE = GetLastError();
    ErrStr.Format("     %s\r\n", (const char *)s);
    ErrStr += GetSYSErrorStr(LE);
    return false;
  };
  m_Stat_ErrFrame = 0;
  m_Stat_Break = 0;

  ZeroMemory(&m_dcb, sizeof(m_dcb));

  m_dcb.DCBlength = sizeof(m_dcb);
  //Speed = 1000000;
		m_dcb.BaudRate = Speed; ///2; 
  //m_dcb.BaudRate = 128000;    
  //if (EmulMyDevice) m_dcb.BaudRate = 115200;
  m_dcb.fBinary = TRUE;
  m_dcb.fParity = FALSE;
  m_dcb.fDtrControl = DTR_CONTROL_DISABLE;
  m_dcb.fDsrSensitivity = FALSE;
 
  m_dcb.fRtsControl = RTS_CONTROL_ENABLE;
  m_dcb.StopBits = ONESTOPBIT;
  m_dcb.XonLim = 0; //0x800;
  m_dcb.XoffLim = 0; //0x200;
  m_dcb.ByteSize = 8;
  m_dcb.XonChar = 0x11;
  m_dcb.XoffChar = 0x13;
 
  if (!SetCommState(m_hFile, &m_dcb)) 
  {
    ErrStr.Format("       ( %d  ) !", Speed);
    ClosePort();
    return false;
  };

		GetCommState(m_hFile, &m_dcb);
		Con<<"Speed = "<<m_dcb.BaudRate<<"\n";

  SetupComm(m_hFile, 10*1024*1024, 1024);

  
  m_to_Read.ReadIntervalTimeout = 0xffffffff; // A value of MAXDWORD, combined with zero values for both the ReadTotalTimeoutConstant and ReadTotalTimeoutMultiplier members, specifies that the read operation is to return immediately with the bytes that have already been received, even if no bytes have been received
  m_to_Read.ReadTotalTimeoutMultiplier = 0;
  m_to_Read.ReadTotalTimeoutConstant = 0;
  m_to_Read.WriteTotalTimeoutMultiplier = 0; // A value of zero for both the WriteTotalTimeoutMultiplier and WriteTotalTimeoutConstant members indicates that total time-outs are not used for write operations
  m_to_Read.WriteTotalTimeoutConstant = 0;
  if (!SetCommTimeouts(m_hFile, &m_to_Read))
  {
    ErrStr = "   -   ";
    ClosePort();
    return false;
  };
  m_IsWaitStrategy = false;


  m_to_Wait.ReadIntervalTimeout = 1000; 
  m_to_Wait.ReadTotalTimeoutMultiplier = 0;
  m_to_Wait.ReadTotalTimeoutConstant = 1000;
  m_to_Wait.WriteTotalTimeoutMultiplier = 0; // A value of zero for both the WriteTotalTimeoutMultiplier and WriteTotalTimeoutConstant members indicates that total time-outs are not used for write operations
  m_to_Wait.WriteTotalTimeoutConstant = 0;

  
  DWORD M;
  GetCommMask(m_hFile, &M);
  M = 0; //EV_RXCHAR; // A character was received and placed in the input buffer
  SetCommMask(m_hFile, M);

  return true;  
};

bool OSerialPort::SetSpeed(int Speed)
{
  m_dcb.BaudRate = Speed;
  if (!SetCommState(m_hFile, &m_dcb)) 
  {
    return false;
  };
  return true;
};

void OSerialPort::ClosePort()
{
  if (m_hFile == INVALID_HANDLE_VALUE) return;
  CloseHandle(m_hFile);
  m_hFile = INVALID_HANDLE_VALUE;
};

void OSerialPort::SetRTS(bool On)
{
  if (m_hFile == INVALID_HANDLE_VALUE) return;
  m_dcb.fRtsControl = On ? RTS_CONTROL_ENABLE : RTS_CONTROL_DISABLE;
  if (!SetCommState(m_hFile, &m_dcb)) AfxMessageBox("set status - 3!");
};

bool OSerialPort::IsOpened() const
{
  return m_hFile != INVALID_HANDLE_VALUE;  
};


void OSerialPort::WriteData(const BYTE *Buf, int SzBuf)
{
  if (m_hFile == INVALID_HANDLE_VALUE) return;
  //Con<<"WR Sz="<<SzBuf<<"\n";
  DWORD SzWr;
  WriteFile(m_hFile, Buf, SzBuf, &SzWr, NULL);
};

bool OSerialPort::ReadData(BYTE *Buf, int BufSz, int &ReadedSz)
{
 if (m_hFile == INVALID_HANDLE_VALUE) { m_ErrCode = 0; ReadedSz = 0; return false; };

  if (m_IsWaitStrategy) SetWaitStrategy(false);  //      

  BOOL Is = ReadFile(m_hFile, Buf, BufSz, (DWORD *) &ReadedSz, NULL);
  if (!Is) m_ErrCode = GetLastError();
  return !!Is;
};

bool OSerialPort::WaitData(BYTE *Buf, int BufSz, int &ReadedSz, int TimeOut)
{
  ReadedSz = 0;
  m_ErrCode = 0;
  if (m_hFile == INVALID_HANDLE_VALUE) return false; 
  
  if (!m_IsWaitStrategy) SetWaitStrategy(true);  //       -

  const DWORD T0 = GetTickCount();
  do
  {
    int rd = 0;
    BOOL Is = ReadFile(m_hFile, Buf+ReadedSz, BufSz-ReadedSz, (DWORD *) &rd, NULL);
    if (!Is) 
    {
      m_ErrCode = GetLastError();
      return false;
    };
    
    ReadedSz += rd;
    if (ReadedSz>=BufSz) break;

    DWORD T1 = GetTickCount();
    if (int(T1-T0) > TimeOut) break;

    //if (TimeOut<=0) break;
    //if (!m_IsWaitStrategy) 
    //Sleep(1);     
    //SwitchToThread();
    //WaitCommEvent()
    //TimeOut-=1;
    //Con<<"TimeOut = "<<TimeOut<<"\n";

  } while (true);

  return true;
};



void OSerialPort::SetWaitStrategy(bool IsWaitStrategy)
{
  if (m_IsWaitStrategy == IsWaitStrategy) return;
  m_IsWaitStrategy = IsWaitStrategy;
  Con<<"OSerialPort::SetWaitStrategy() - c   "<<int(m_IsWaitStrategy)<<"\n";
  BOOL Is = SetCommTimeouts(m_hFile, m_IsWaitStrategy ? &m_to_Wait : &m_to_Read);
  if (!Is)
  {
    Con<<"OSerialPort::SetWaitStrategy() - call SetCommTimeouts() error !!!! \n";
  };
};

BOOL OSerialPort::ClearCommError(LPDWORD pComErrs, LPCOMSTAT pComStat)
{
  BOOL IsCE = ::ClearCommError(m_hFile, pComErrs, pComStat);
  if (IsCE)
  {
    if (*pComErrs & CE_FRAME) m_Stat_ErrFrame++;
    if (*pComErrs & CE_BREAK) m_Stat_Break++;
  };
  return IsCE;
};
    

BOOL OSerialPort::ClrBreak()
{
  return EscapeCommFunction(m_hFile, CLRBREAK);
};

BOOL OSerialPort::SetBreak()
{
  return EscapeCommFunction(m_hFile, SETBREAK);
};

BOOL OSerialPort::SetDTR()
{
  return EscapeCommFunction(m_hFile, SETDTR);
};

BOOL OSerialPort::ClrDTR()
{
  return EscapeCommFunction(m_hFile, CLRDTR);
};

BOOL OSerialPort::SetRTS()
{
  return EscapeCommFunction(m_hFile, SETRTS);
};

BOOL OSerialPort::ClrRTS()
{
  return EscapeCommFunction(m_hFile, CLRRTS);
};

